%% 1D Linear Advection Equation
%     .
%     T + v T = 0   for T=T(x,t), 0<x<L, t>0
%
% Initial value/boundary value problem with 
%    T(x,0) = Tinit(x)  (initial condition)             
%    T(0,t) = 0, T(L,t) = 0 for all t>0  (boundary conditions)


%% Discretization of the domain
L = 2;                      %  length of spatial domain (in m)
tend = 0.96;                %  time interval from t=0 to t=tend

n=250;                      %  n equidistant interior gridpoints
h=1/(n+1);                  %  grid width
x=linspace(0,L,n+2);        %  grid vector including boundary points

%% System parameters
v = 1; % transport velocity in m/s 

% relevant dimensionless parameters are 
% 
% * Courant number (flow distance, compared to grid size)
% * Fourier number (diffusive transport, compared to accumulation in cell)

% in this version, we specify C and calculate time step from it
C = 0.8; % set the Courant Number
dt = C*h/v;

%% Time loop
% initialize T with triangular spike
T = spike(x/L);
axis([0 L -0.1 1]);
t=0;


while (t< tend-0.5*dt)
    % explicit Upwind scheme, no diffusion: Stencil
    %                    1                 
    %          C    ... 1-C  ...  0
    
     T(2:n+1) = C*T(1:n) + (1-C)*T(2:n+1);
    
    % Lax-Wendroff scheme: stencil
    %                          1                 
    %          C/2+C^2/2 ... 1-C^2  ... -C/2+C^2/2
    % T(2:n+1) = (C+C^2)/2*T(1:n) + (1-C^2)*T(2:n+1) + (-C+C^2)/2*T(3:n+2);
    %T(2:n+1) = (1+C)/2*T(1:n) + (0)*T(2:n+1) + (1-C)/2*T(3:n+2);
    
    % more general: convection diffusion upwind stencil
    %                    1                 
    %     C+lam  ...  1-C-2*lam  ...  lam
    %
    % note stability condition: 1-C-2*lam >=0
    % T(2:n+1) = (C+lam)*T(1:n) + (1-C-2*lam)*T(2:n+1) + lam*T(3:n+2);
    
    t = t+dt;
    plot(x,T);axis([0 L -0.1 1]);
    pause(0.01)
end
